package com.sd.business.service.inventory;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.service.BaseService;
import com.sd.business.entity.inventory.dto.InventoryFinishedErpImportDataDto;
import com.sd.business.entity.inventory.dto.InventoryFinishedOrderSelectDto;
import com.sd.business.entity.inventory.po.InventoryFinishedOrder;
import com.sd.business.entity.inventory.vo.InventoryFinishedOrderVo;
import com.sd.business.entity.order.po.OrderSku;

import java.util.List;


/**
 * <p>
 * 成品仓库订单 服务类
 * </p>
 *
 * @author
 * @since 2023-10-10
 */
public interface InventoryFinishedOrderService extends BaseService<InventoryFinishedOrder> {

    /**
     * 生产入库
     *
     * @param orderSkuList 订单明细列表
     */
    void productionWarehousing(List<OrderSku> orderSkuList);

    /**
     * 根据订单id删除记录
     *
     * @param orderId 订单id
     */
    void removeByOrderId(Long orderId);

    /**
     * 根据id删除记录
     *
     * @param ids
     */
    void removeByIds(List<Long> ids);

    /**
     * 销售出库
     *
     * @param inventoryFinishedOrderList 成品仓库订单明细
     */
    void saleOutOfWarehouse(List<InventoryFinishedOrder> inventoryFinishedOrderList);

    /**
     * 成品仓库订单分页
     */
    Page<InventoryFinishedOrderVo> getPage(InventoryFinishedOrderSelectDto dto);

    /**
     * excel导出
     */
    void excelExport(InventoryFinishedOrderSelectDto dto);

    /**
     * 成品仓明细E10模板导出
     * @param list
     */
    void erpExcelExport(List<InventoryFinishedErpImportDataDto> list);

    /**
     * 无源库存销售出库
     *
     * @param orderSkuList
     */
    void noSourceSaleOutOfWarehouse(List<OrderSku> orderSkuList);

    /**
     * 根据订单id列表查询没有销货出库的订单
     */
    List<Long> getNotSalesOutOfWarehouseOrderByIds(List<Long> orderIdList);

    /**
     * 同步昨日无理由订单成品入库
     */
    void syncYesterdayNoReasonOrderInventory();
}
