package com.sd.business.service.inventory.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.inventory.dto.InventoryFinishedOrderInStorageDto;
import com.sd.business.entity.inventory.dto.InventoryFinishedSelectDto;
import com.sd.business.entity.inventory.enums.FinishedOperationTypeEnum;
import com.sd.business.entity.inventory.po.InventoryFinished;
import com.sd.business.entity.inventory.po.InventoryFinishedOrder;
import com.sd.business.entity.inventory.po.InventoryFinishedOrderDetail;
import com.sd.business.entity.inventory.vo.InventoryFinishedVo;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.mapper.inventory.InventoryFinishedMapper;
import com.sd.business.service.inventory.InventoryFinishedOrderDetailService;
import com.sd.business.service.inventory.InventoryFinishedOrderService;
import com.sd.business.service.inventory.InventoryFinishedService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.framework.util.excel.util.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;


/**
 * <p>
 * 成品仓库 服务实现类
 * </p>
 *
 * @author
 * @since 2023-10-10
 */
@Service
public class InventoryFinishedServiceImpl extends ServiceImpl<InventoryFinishedMapper, InventoryFinished> implements InventoryFinishedService {

    @Autowired
    private HttpServletResponse response;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private InventoryFinishedOrderService inventoryFinishedOrderService;

    @Autowired
    private InventoryFinishedOrderDetailService inventoryFinishedOrderDetailService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private OrderService orderService;

    @Override
    public Page<InventoryFinishedVo> getPage(InventoryFinishedSelectDto dto) {
        IWrapper<InventoryFinished> wrapper = getWrapper();
        wrapper.orderByDesc("i", InventoryFinished::getId);
        wrapper.like("ss", SkuSpec::getCode, dto.getSkuSpecCode());
        wrapper.like("ss", SkuSpec::getName, dto.getSkuSpecName());
        wrapper.gt("i", InventoryFinished::getQuantity, 0);
        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void productionWarehousing(List<Long> orderIdList) {
        if (orderIdList.isEmpty()) {
            return;
        }

        // 查询订单sku明细
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));
        if (orderSkuList.isEmpty()) {
            return;
        }

        // 筛选掉赠品sku
        List<Long> skuSpecIds = orderSkuList.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Set<Long> giftSkuSpecIds = skuSpecService.getGiftIdListByIdList(skuSpecIds);
        List<OrderSku> list = orderSkuList.stream()
                .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                .collect(Collectors.toList());

        // 添加成品仓库存
        List<InventoryFinished> inventoryFinishedList = list.stream().map(item -> {
            InventoryFinished inventoryFinished = new InventoryFinished();
            inventoryFinished.setSkuSpecId(item.getSkuSpecId());
            inventoryFinished.setQuantity(item.getQuantity());
            return inventoryFinished;
        }).collect(Collectors.toList());
        inOut(inventoryFinishedList, true);

        inventoryFinishedOrderService.productionWarehousing(list);
    }

    @Override
    public void removeOrder(Long orderId) {

        // 查询成品仓库
        List<InventoryFinishedOrder> list = inventoryFinishedOrderService.list(q -> q
                .eq(InventoryFinishedOrder::getOrderInfoId, orderId)
                .eq(InventoryFinishedOrder::getStatus, StatusConstant.YES));

        if (list.isEmpty()) {
            return;
        }

        // 成品库存出库
        outByFinishedOrder(list);

        List<Long> inventoryFinishedOrderIds = list.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        // 删除成品仓库订单以及明细
        inventoryFinishedOrderService.removeByIds(inventoryFinishedOrderIds);
    }

    @Override
    public void saleOutOfWarehouse(List<Long> orderIdList) {
        if (orderIdList.isEmpty()) {
            return;
        }

        // 查询成品仓库
        List<InventoryFinishedOrder> list = inventoryFinishedOrderService.list(q -> q
                .in(InventoryFinishedOrder::getOrderInfoId, orderIdList)
                .eq(InventoryFinishedOrder::getStatus, StatusConstant.YES));

        if (list.isEmpty()) {
            return;
        }

        // 成品库存出库
        outByFinishedOrder(list);

        // 产品库订单出库
        inventoryFinishedOrderService.saleOutOfWarehouse(list);
    }

    @Override
    public void excelExport(InventoryFinishedSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<InventoryFinishedVo> page = getPage(dto);
        List<InventoryFinishedVo> records = page.getRecords();

        ExcelUtil.export(response, records, InventoryFinishedVo.class);
    }

    /**
     * 成品仓sku规格出入库
     */
    private void inOut(List<InventoryFinished> list, boolean isIn) {

        if (list.isEmpty()) {
            return;
        }

        // 相同sku合并
        Map<Long, InventoryFinished> map = list.stream().collect(Collectors.toMap(
                InventoryFinished::getSkuSpecId,
                Function.identity(),
                (v1, v2) -> {
                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                    if (v1.getLockQuantity() != null && v2.getLockQuantity() != null) {
                        v1.setLockQuantity(v1.getLockQuantity().add(v2.getLockQuantity()));
                    }
                    return v1;
                }));

        synchronized (this) {

            // 获取原库存数据
            Map<Long, InventoryFinished> oldStorageMap = mapKEntity(InventoryFinished::getSkuSpecId,
                    q -> q.in(InventoryFinished::getSkuSpecId, map.keySet()));

            // 计算入库后库存数量
            for (InventoryFinished item : map.values()) {
                InventoryFinished inventoryFinished = oldStorageMap.get(item.getSkuSpecId());
                if (inventoryFinished == null) {
                    continue;
                }
                BigDecimal quantity;
                if (isIn) {
                    quantity = inventoryFinished.getQuantity().add(item.getQuantity());
                } else {
                    quantity = inventoryFinished.getQuantity().subtract(item.getQuantity());
                    // 锁定数量更新
                    if (item.getLockQuantity() != null) {
                        inventoryFinished.setLockQuantity(inventoryFinished.getLockQuantity().subtract(item.getLockQuantity()));
                    }
                }
                item.setId(inventoryFinished.getId());
                item.setLockQuantity(inventoryFinished.getLockQuantity());
                item.setQuantity(quantity);
            }

            // 更新数据库
            saveOrUpdateBatch(map.values());
        }

    }

    /**
     * 成品库存出库
     *
     * @param list 成品仓库
     */
    private void outByFinishedOrder(List<InventoryFinishedOrder> list) {

        List<InventoryFinished> inventoryFinishedList = list.stream().map(item -> {
            InventoryFinished inventoryFinished = new InventoryFinished();
            inventoryFinished.setQuantity(item.getQuantity());
            inventoryFinished.setSkuSpecId(item.getSkuSpecId());
            return inventoryFinished;
        }).collect(Collectors.toList());

        inOut(inventoryFinishedList, false);

    }

    /**
     * 锁定无源成品库存
     * @param orderSkuList
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean lockNoSourceStorage(List<? extends OrderSku> orderSkuList) {
        if (ObjectUtil.isEmpty(orderSkuList)) {
            return true;
        }

        Map<Long, BigDecimal> map = orderSkuList.stream()
                .collect(Collectors.toMap(OrderSku::getSkuSpecId, OrderSku::getQuantity, BigDecimal::add));

        synchronized (this) {

            List<InventoryFinished> list = list(q -> q.in(InventoryFinished::getSkuSpecId, map.keySet()));

            if (map.size() != list.size()) {
                return false;
            }

            // 查询无源成品数据
            List<InventoryFinishedOrder> inventoryFinishedOrderList = inventoryFinishedOrderService.list(q -> q
                    .in(InventoryFinishedOrder::getSkuSpecId, map.keySet())
                    .eq(InventoryFinishedOrder::getStatus, 1)
                    .isNull(InventoryFinishedOrder::getOrderInfoId));
            Map<Long, BigDecimal> inventoryFinishedMap = inventoryFinishedOrderList.stream()
                    .collect(Collectors.toMap(
                            InventoryFinishedOrder::getSkuSpecId,
                            InventoryFinishedOrder::getExistingQuantity,
                            BigDecimal::add));

            if (map.size() != inventoryFinishedMap.size()) {
                return false;
            }

            for (InventoryFinished inventoryFinished : list) {
                BigDecimal lockQuantity = map.get(inventoryFinished.getSkuSpecId());
                // 查询成品sku现存数量，减去锁定库存，得出当前可用数量
                BigDecimal existingQuantity = inventoryFinishedMap.getOrDefault(inventoryFinished.getSkuSpecId(), BigDecimal.ZERO);
                if (existingQuantity.subtract(inventoryFinished.getLockQuantity()).compareTo(lockQuantity) >= 0) {
                    inventoryFinished.setLockQuantity(inventoryFinished.getLockQuantity().add(lockQuantity));
                } else {
                    return false;
                }
            }

            updateBatchById(list);
        }

        return true;
    }

    /**
     * 解锁无源成品库存
     * @param orderSkuList
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void unlockNoSourceStorage(List<OrderSku> orderSkuList) {
        if (ObjectUtil.isEmpty(orderSkuList)) {
            return;
        }

        Map<Long, BigDecimal> map = orderSkuList.stream()
                .collect(Collectors.toMap(OrderSku::getSkuSpecId, OrderSku::getQuantity, BigDecimal::add));

        synchronized (this) {

            List<InventoryFinished> list = list(q -> q.in(InventoryFinished::getSkuSpecId, map.keySet()));

            for (InventoryFinished inventoryFinished : list) {
                BigDecimal lockQuantity = map.get(inventoryFinished.getSkuSpecId());
                inventoryFinished.setLockQuantity(inventoryFinished.getLockQuantity().subtract(lockQuantity));
            }

            updateBatchById(list);
        }
    }

    /**
     * 无源成品销售出库
     * @param orderIdList 订单id列表
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void noSourceSaleOutOfWarehouse(List<Long> orderIdList) {
        if (ObjectUtil.isEmpty(orderIdList)) {
            return;
        }

        // 查询订单sku明细
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));
        if (orderSkuList.isEmpty()) {
            return;
        }

        // 筛选掉赠品sku
        List<Long> skuSpecIds = orderSkuList.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Set<Long> giftSkuSpecIds = skuSpecService.getGiftIdListByIdList(skuSpecIds);
        List<OrderSku> list = orderSkuList.stream()
                .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                .collect(Collectors.toList());

        // 成品库存出库
        List<InventoryFinished> inventoryFinishedList = list.stream().map(item -> {
            InventoryFinished inventoryFinished = new InventoryFinished();
            inventoryFinished.setQuantity(item.getQuantity());
            inventoryFinished.setSkuSpecId(item.getSkuSpecId());
            inventoryFinished.setLockQuantity(item.getQuantity());
            return inventoryFinished;
        }).collect(Collectors.toList());
        inOut(inventoryFinishedList, false);

        // 产品库订单出库
        inventoryFinishedOrderService.noSourceSaleOutOfWarehouse(list);
    }

    /**
     * 无源退料入库
     *
     * @param orderSkuList
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void noSourceInWarehousing(List<OrderSku> orderSkuList) {
        if (ObjectUtil.isEmpty(orderSkuList)) {
            return;
        }
        // 筛选掉赠品sku
        List<Long> skuSpecIds = orderSkuList.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Set<Long> giftSkuSpecIds = skuSpecService.getGiftIdListByIdList(skuSpecIds);
        List<OrderSku> list = orderSkuList.stream()
                .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                .collect(Collectors.toList());

        // 添加成品仓库存
        List<InventoryFinished> inventoryFinishedList = list.stream().map(item -> {
            InventoryFinished inventoryFinished = new InventoryFinished();
            inventoryFinished.setSkuSpecId(item.getSkuSpecId());
            inventoryFinished.setQuantity(item.getQuantity());
            return inventoryFinished;
        }).collect(Collectors.toList());
        inOut(inventoryFinishedList, true);

        // 清空来源信息
        list.forEach(item -> {
            item.setId(null);
            item.setOrderId(null);
        });

        inventoryFinishedOrderService.productionWarehousing(list);
    }

    @Override
    public List<InventoryFinishedVo> getTodayInStorageList() {
        Date date = DateUtil.beginOfDay(new Date());
        // 查询今日成品入库明细
        List<InventoryFinishedOrderDetail> finishedOrderDetailList = inventoryFinishedOrderDetailService.list(q -> q
                .eq(InventoryFinishedOrderDetail::getOperationType, 1)
                .isNull(InventoryFinishedOrderDetail::getOrderInfoId)
                .ge(BasePo::getCreateTime, date));

        // 获取无理由订单
        List<OrderInfo> orderInfoList = orderService.list(q -> q
                .eq(OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey())
                .ge(OrderInfo::getShippingTime, date));
        List<Long> orderIds = orderInfoList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        List<InventoryFinishedOrder> inventoryFinishedOrderList = new ArrayList<>();
        if (!orderIds.isEmpty()) {
            inventoryFinishedOrderList = inventoryFinishedOrderService.list(q -> q
                    .in(InventoryFinishedOrder::getOrderInfoId, orderIds)
                    .eq(InventoryFinishedOrder::getStatus, StatusConstant.YES));
        }

        Map<Long, InventoryFinishedVo> inventoryFinishedVoMap = Stream.concat(
                finishedOrderDetailList.stream().map(item -> {
                    InventoryFinishedVo vo = new InventoryFinishedVo();
                    vo.setSkuSpecId(item.getSkuSpecId());
                    vo.setQuantity(item.getQuantity());
                    return vo;
                }),
                inventoryFinishedOrderList.stream().map(item -> {
                    InventoryFinishedVo vo = new InventoryFinishedVo();
                    vo.setSkuSpecId(item.getSkuSpecId());
                    vo.setQuantity(item.getQuantity());
                    return vo;
                })
        ).collect(Collectors.toMap(
                InventoryFinishedVo::getSkuSpecId,
                Function.identity(),
                (v1, v2) -> {
                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                    return v1;
                })
        );

        List<InventoryFinishedVo> list = new ArrayList<>(inventoryFinishedVoMap.values());
        // 赋值sku品号品名
        skuSpecService.attributeAssign(list, InventoryFinishedVo::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecCode(skuSpec.getCode());
            item.setSkuSpecName(skuSpec.getName());
        });

        return list;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void inStorageAdd(InventoryFinishedOrderInStorageDto inStorageDto) {
        List<InventoryFinishedOrder> skuList = inStorageDto.getSkuList();

        // 添加成品仓库存
        List<InventoryFinished> inventoryFinishedList = skuList.stream().map(item -> {
            InventoryFinished inventoryFinished = new InventoryFinished();
            inventoryFinished.setSkuSpecId(item.getSkuSpecId());
            inventoryFinished.setQuantity(item.getQuantity());
            return inventoryFinished;
        }).collect(Collectors.toList());
        inOut(inventoryFinishedList, true);

        for (InventoryFinishedOrder inventoryFinishedOrder : skuList) {
            inventoryFinishedOrder.setDepartmentId(inStorageDto.getDepartmentId());
            inventoryFinishedOrder.setStatus(StatusConstant.YES);
            inventoryFinishedOrder.setExistingQuantity(inventoryFinishedOrder.getQuantity());
            inventoryFinishedOrder.setRemark(inStorageDto.getRemark());
        }
        // 新增入库记录
        inventoryFinishedOrderService.saveBatch(skuList);
        // 添加流水记录
        inventoryFinishedOrderDetailService.add(skuList, FinishedOperationTypeEnum.PRODUCTION_WAREHOUSING);
    }
}
