package com.sd.business.service.order.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.ruoyi.common.annotation.LogicIgnore;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserService;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.bo.BomSpecOrderConsumptionBo;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.in.dto.InOutStorageDto;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.emums.OutDetailTypeEnum;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.order.dto.*;
import com.sd.business.entity.order.enums.OrderClassifyEnum;
import com.sd.business.entity.order.enums.OrderExceptionTypeEnum;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.*;
import com.sd.business.entity.order.vo.*;
import com.sd.business.entity.price.po.PriceBillingStandard;
import com.sd.business.entity.price.po.PriceBillingStandardDetail;
import com.sd.business.entity.production.po.ProductionWorkOrder;
import com.sd.business.entity.production.vo.OutBomVo;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.sku.po.SkuSpecLink;
import com.sd.business.entity.statement.po.StatementOfAccount;
import com.sd.business.entity.statement.vo.StatementOrderClassifyTotalCountVo;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.order.OrderMapper;
import com.sd.business.service.bom.BomService;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.business.service.inventory.InventoryFinishedService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.order.*;
import com.sd.business.service.price.PriceBillingStandardDetailService;
import com.sd.business.service.price.PriceBillingStandardService;
import com.sd.business.service.production.ProductionWorkOrderService;
import com.sd.business.service.sku.SkuSpecLinkService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.business.util.CodeEnum;
import com.sd.framework.util.Assert;
import com.sd.framework.util.excel.util.ExcelUtil;
import com.sd.mq.config.ArtworkConfig;
import com.sd.mq.entity.TempArtworkMessage;
import com.sd.mq.util.RabbitMqUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 订单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-06-30
 */
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, OrderInfo> implements OrderService {

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private BomService bomService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private OrderPackageBomService orderPackageBomService;

    @Autowired
    private OrderOperatingLogService orderOperatingLogService;

    @Autowired
    private PriceBillingStandardService priceBillingStandardService;

    @Autowired
    private PriceBillingStandardDetailService priceBillingStandardDetailService;

    @Autowired
    private InOutStorageService inOutStorageService;

    @Autowired
    private StatementOfAccountService statementOfAccountService;

    @Autowired
    private InventoryService inventoryService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private OrderSkuProductionCostService orderSkuProductionCostService;

    @Autowired
    private InventoryFinishedService inventoryFinishedService;

    @Autowired
    private ProductionWorkOrderService productionWorkOrderService;

    @Autowired
    private SkuSpecLinkService skuSpecLinkService;

    @Autowired
    private HttpServletResponse response;

    @Override
    public Page<OrderInfoVo> getPage(OrderSelectDto dto) {
        IWrapper<OrderInfo> wrapper = getWrapper();
        wrapper.orderByDesc("if(o.status = 60, 1, 0)");
        wrapper.orderByDesc("o", OrderInfo::getId);
        wrapper.eq("d", Department::getId, dto.getDepartmentId());
        wrapper.like("d", Department::getName, dto.getDepartmentName());
        wrapper.like("o", OrderInfo::getCode, dto.getCode());
        wrapper.like("o", OrderInfo::getWlnCode, dto.getWlnCode());
        wrapper.eq("o", OrderInfo::getStatus, dto.getStatus());
        wrapper.eq("o", OrderInfo::getSettlementStatus, dto.getSettlementStatus());
        wrapper.ge("o", OrderInfo::getWlnCreateTime, dto.getBeginTime());
        wrapper.le("o", OrderInfo::getWlnCreateTime, dto.getEndTime());
        wrapper.eq("o", OrderInfo::getFlowStatus, dto.getFlowStatus());
        wrapper.and(q -> q
                .and(r -> r.eq("o", OrderInfo::getWlnStorageCode, "T007").ge("o", OrderInfo::getWlnStatus, 2))
                .or(r -> r.ne("o", OrderInfo::getWlnStorageCode, "T007").ge("o", OrderInfo::getWlnStatus, 1))
                .or(r -> r.eq("o", OrderInfo::getSource, 1))
        );

        List<Long> detpIdList = sysUserService.getDetpIdListByUserId(SecurityUtils.getUserId());
        if (!detpIdList.isEmpty()) {
            wrapper.in("o", OrderInfo::getDepartmentId, detpIdList);
            wrapper.eq("o", OrderInfo::getCreateUser, SecurityUtils.getUserId());
        }

        if (Objects.equals(dto.getLinkedStatementOfAccount(), 0)) {
            wrapper.isNull("o.statement_of_account_id");
            wrapper.in("o", OrderInfo::getStatus,
                    OrderStatusEnum.COMPLETION_PRODUCTION.getKey(),
                    OrderStatusEnum.HAVE_BEEN_SHIPPED.getKey());
        } else if (Objects.equals(dto.getLinkedStatementOfAccount(), 1)) {
            wrapper.isNotNull("o.statement_of_account_id");
            wrapper.in("o", OrderInfo::getStatus,
                    OrderStatusEnum.COMPLETION_PRODUCTION.getKey(),
                    OrderStatusEnum.HAVE_BEEN_SHIPPED.getKey());
        }

        if (dto.getException() != null) {
            if (dto.getException().equals(StatusConstant.YES)) {
                wrapper.ne("o", OrderInfo::getExceptionType, "0");
            } else {
                wrapper.eq("o", OrderInfo::getExceptionType, "0");
            }
        }

        wrapper.apply(dto.getExceptionType() != null,
                "find_in_set({0}, o.exception_type)", dto.getExceptionType());

        Page<OrderInfoVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<OrderInfoVo> records = page.getRecords();

        if (records.isEmpty()) {
            return page;
        }

        for (OrderInfoVo record : records) {

            // 设置异常提示
            String exceptionType = record.getExceptionType();
            if (exceptionType.equals("0")) {
                continue;
            }
            String exceptionTypeDetail = Arrays.stream(exceptionType.split(","))
                    .map(Convert::toInt)
                    .map(OrderExceptionTypeEnum::getEnumValue)
                    .collect(Collectors.joining(","));
            record.setExceptionTypeDetail(exceptionTypeDetail);
        }

        return page;
    }

    @Override
    public OrderInfoVo detail(Long id) {
        OrderInfo orderInfo = this.getById(id);
        if (orderInfo == null) {
            throw new ServiceException("未知订单");
        }

        // 订单
        OrderInfoVo result = BeanUtil.toBean(orderInfo, OrderInfoVo.class);

        // 赋值事业部名称
        Department department = departmentService.getById(orderInfo.getDepartmentId());
        if (department != null) {
            result.setDepartmentName(department.getName());
        }

        // 赋值订单明细
        List<OrderSku> tempOrderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<OrderSkuVo> orderSkuList = BeanUtil.copyToList(tempOrderSkuList, OrderSkuVo.class);

        skuSpecService.attributeAssign(orderSkuList, OrderSkuVo::getSkuSpecId, (item, skuSpec) -> {
            item.setName(skuSpec.getName());
            item.setCode(skuSpec.getCode());
        });
        bomSpecService.attributeAssign(orderSkuList, OrderSkuVo::getBomSpecId, (item, bomSpec) -> {
            item.setBomName(bomSpec.getName());
            item.setBomCode(bomSpec.getCode());
        });
        result.setOrderSkuList(orderSkuList);

        // 赋值包材配件/单品
        List<OrderSkuBom> tempOrderSkuBomList = orderSkuBomService.list(q -> q.eq(OrderSkuBom::getOrderId, id));
        List<OrderSkuBomVo> orderSkuBomList = BeanUtil.copyToList(tempOrderSkuBomList, OrderSkuBomVo.class);
        List<Long> bomSpecIds = orderSkuBomList.stream().map(OrderSkuBom::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIds);
        orderSkuBomList.forEach(item -> {
            BomSpecBo bomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
            if (bomSpecBo == null) {
                return;
            }
            item.setBomSpecCode(bomSpecBo.getBomSpecCode());
            item.setBomSpecName(bomSpecBo.getBomSpecName());
            item.setBomClassifyId(bomSpecBo.getClassifyId());
        });

        Map<Long, List<OrderSkuBomVo>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        for (OrderSkuVo orderSkuVo : orderSkuList) {
            orderSkuVo.setOrderSkuBomList(orderSkuBomMap.getOrDefault(orderSkuVo.getId(), Collections.emptyList()));
        }

        // 赋值包装明细
        List<OrderPackageBom> list = orderPackageBomService.list(q -> q.eq(OrderPackageBom::getOrderId, id));

        List<OrderInfoVo.OrderPackage> orderPackageList = list.stream().map(item -> {
            OrderInfoVo.OrderPackage orderPackage = new OrderInfoVo.OrderPackage();
            orderPackage.setBomSpecId(item.getBomSpecId());
            orderPackage.setQuantity(item.getQuantity());
            orderPackage.setCostPrice(item.getCostPrice());
            orderPackage.setInternalSellingPrice(item.getInternalSellingPrice());
            return orderPackage;
        }).collect(Collectors.toList());
        // 赋值品名 品号 销售单价 成本价
        bomSpecService.attributeAssign(orderPackageList, OrderInfoVo.OrderPackage::getBomSpecId, (item, bomSpec) -> {
            item.setCode(bomSpec.getCode());
            item.setName(bomSpec.getName());
        });
        result.setOrderPackageBomList(orderPackageList);

        return result;
    }

    @DSTransactional
    @Override
    public void add(OrderInfoDto orderDto) {

        List<OrderSkuDto> orderSkuList = orderDto.getOrderSkuList();
        if (ObjectUtil.equal(orderDto.getStatus(), OrderStatusEnum.STOCK_PREPARATION.getKey())
                && ObjectUtil.notEqual(orderDto.getType(), 2)) {
            // 先判断成品是否能锁定
            if (inventoryFinishedService.lockNoSourceStorage(orderSkuList)) {
                orderDto.setStockType(StatusConstant.YES);
            } else {
                lockStorage(orderSkuList);
                orderDto.setStockType(StatusConstant.NO);
            }
            orderDto.setLockStorage(StatusConstant.YES);
        } else {
            orderDto.setLockStorage(StatusConstant.NO);
        }

        // 新增订单
        orderDto.setSource(1);
        orderDto.setCode("PI" + new Date().getTime());
        orderDto.setSettlementStatus(1);
        orderDto.setTaxRate(new BigDecimal("10.00"));
        orderDto.setTag("0");
        orderDto.setExceptionType(OrderExceptionTypeEnum.NORMAL.getKey().toString());
        if (Objects.equals(orderDto.getType(), 2)) {
            orderDto.setClassify(OrderClassifyEnum.OUTSOURCE_ORDER.getKey());
            // 委外订单修改金额
            orderDto.setProductTotalAmount(BigDecimal.ZERO);
            orderDto.setDeliveryMaterialsFee(BigDecimal.ZERO);
            orderDto.setPackingLabor(orderDto.getPackingLabor().multiply(new BigDecimal(2)));
            orderDto.setTotalAmount(orderDto.getProductTotalAmount()
                    .add(orderDto.getCustomProcessingFee())
                    .add(orderDto.getLssueFee())
                    .add(orderDto.getDeliveryMaterialsFee())
                    .add(orderDto.getPackingLabor())
                    .add(orderDto.getPackagingMaterialCost())
                    .add(orderDto.getProofingFee())
                    .add(orderDto.getManagementFee()));
        } else if (Objects.equals(orderDto.getType(), 1)) {
            // 拥有采购角色的账号创建的自主订单为采购订单
            Long userId = SecurityUtils.getUserId();
            Set<String> roleKeys = roleService.selectRolePermissionByUserId(userId);
            if (!Collections.disjoint(roleKeys, Arrays.asList("sypurchasing", "purchasingOfficer", "bzpurchasing"))) {
                orderDto.setClassify(OrderClassifyEnum.PURCHASE_ORDER.getKey());
            }
        }
        this.save(orderDto);

        // 新增订单产品包材配件
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(orderDto.getId()))
                .peek(item -> item.setStockPreparationStatus(StatusConstant.NO))
                .peek(item -> {
                    // 委外订单修改金额
                    if (ObjectUtil.equals(orderDto.getType(), 2)) {
                        item.setUnitPrice(BigDecimal.ZERO);
                        item.setLssueFee(BigDecimal.ZERO);
                        item.setDeliveryMaterialsFee(BigDecimal.ZERO);
                        item.setPackingLabor(item.getPackingLabor().multiply(new BigDecimal(2)));
                    }
                })
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.saveBatch(tempOrderSkuList);

        // 新增订单产品包装信息
        List<OrderSkuBom> orderSkuBomList = orderSkuList.stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(orderDto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
        orderSkuBomService.saveBatch(orderSkuBomList);

        // 新增订单产品生产成本
        Map<Long, List<OrderSkuBom>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        List<OrderSkuProductionCost> orderSkuProductionCostList = tempOrderSkuList.stream().map(item -> {
            OrderSkuProductionCost orderSkuProductionCost = new OrderSkuProductionCost();
            orderSkuProductionCost.setOrderId(item.getOrderId());
            orderSkuProductionCost.setOrderSkuId(item.getId());
            // 材料成本
            orderSkuProductionCost.setMaterialCost(item.getUnitPrice().multiply(item.getQuantity()));
            // 辅料成本暂时没有默认为零
            orderSkuProductionCost.setAuxiliaryMaterialCost(BigDecimal.ZERO);
            // 产品包材成本
            List<OrderSkuBom> bomList = orderSkuBomMap.get(item.getId());
            if (bomList == null) {
                bomList = new ArrayList<>();
            }
            BigDecimal productPackagingMaterialCost = bomList.stream()
                    .map(orderSkuBom -> orderSkuBom.getQuantity()
                            .multiply(orderSkuBom.getUnitPrice())
                            .multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            orderSkuProductionCost.setProductPackagingMaterialCost(productPackagingMaterialCost);
            // 物流包材成本
            orderSkuProductionCost.setLogisticsPackagingMaterialCost(item.getDeliveryMaterialsFee()
                    .multiply(item.getQuantity()));
            orderSkuProductionCost.setTotalAmount(
                    orderSkuProductionCost.getMaterialCost()
                            .add(orderSkuProductionCost.getAuxiliaryMaterialCost())
                            .add(orderSkuProductionCost.getProductPackagingMaterialCost())
                            .add(orderSkuProductionCost.getLogisticsPackagingMaterialCost()));
            return orderSkuProductionCost;
        }).collect(Collectors.toList());
        orderSkuProductionCostService.saveBatch(orderSkuProductionCostList);

        // 是委外订单并且没有包材时，修改订单状态为生产中，并生成生产任务和工单
        if (ObjectUtil.notEqual(orderDto.getStatus(), OrderStatusEnum.DRAFT.getKey())
                && ObjectUtil.equal(orderDto.getFlowStatus(), FlowStatusEnum.PASS.getKey())
                && ObjectUtil.equals(orderDto.getType(), 2)
                && ObjectUtil.isEmpty(orderSkuBomList)) {
            orderDto.setStatus(OrderStatusEnum.IN_PRODUCTION.getKey());
            // 生成工单
            productionWorkOrderService.add(orderDto.getId());
            this.update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey()).eq(BaseIdPo::getId, orderDto.getId()));
        }

        // 保存订单包装信息
        if (ObjectUtil.isNotEmpty(orderDto.getOrderPackageBomList())) {
            List<OrderPackageBomDto> tempOrderPackageBomList = orderDto.getOrderPackageBomList();
            List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                    .peek(item -> item.setOrderId(orderDto.getId()))
                    .map(item -> (OrderPackageBom) item)
                    .collect(Collectors.toList());
            orderPackageBomService.saveBatch(orderPackageBomList);
        }

        // 订单操作日志
        OrderOperatingLog orderOperatingLog = new OrderOperatingLog();
        orderOperatingLog.setType(10);
        orderOperatingLog.setOrderId(orderDto.getId());
        orderOperatingLog.setOrderCode(orderDto.getCode());
        orderOperatingLogService.save(orderOperatingLog);

        // 保存产品不干胶图稿
        for (OrderSkuDto item : orderSkuList) {
            ObsFileUtil.saveFile(item.getSelfAdhesiveStickerFile(), item.getId());
        }

        // 保存附件
        ObsFileUtil.saveFile(orderDto.getFileList(), orderDto.getId());

    }

    @DSTransactional
    @Override
    public void edit(OrderInfoDto dto) {
        // 订单为委外订单时，订单分类为委外订单
        if (Objects.equals(dto.getType(), 2)) {
            dto.setClassify(OrderClassifyEnum.OUTSOURCE_ORDER.getKey());
        } else if (Objects.equals(dto.getType(), 1)) {
            // 拥有采购角色的账号创建的自主订单为采购订单
            Long userId = dto.getCreateUser();
            Set<String> roleKeys = roleService.selectRolePermissionByUserId(userId);
            if (!Collections.disjoint(roleKeys, Arrays.asList("sypurchasing", "purchasingOfficer", "bzpurchasing"))) {
                dto.setClassify(OrderClassifyEnum.PURCHASE_ORDER.getKey());
            }
        }

        // 更新订单
        this.updateById(dto);

        // 更新订单产品
        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .peek(item -> item.setStockPreparationStatus(StatusConstant.NO))
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.editLinked(tempOrderSkuList, OrderSku::getOrderId, dto.getId());

        // 更新订单产品包材配件
        List<OrderSkuBom> orderSkuBomList = orderSkuList.stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(dto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
        orderSkuBomService.editLinked(orderSkuBomList, OrderSkuBom::getOrderId, dto.getId());

        // 新增订单产品生产成本
        Map<Long, List<OrderSkuBom>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        List<OrderSkuProductionCost> orderSkuProductionCostList = tempOrderSkuList.stream().map(item -> {
            OrderSkuProductionCost orderSkuProductionCost = new OrderSkuProductionCost();
            orderSkuProductionCost.setOrderId(item.getOrderId());
            orderSkuProductionCost.setOrderSkuId(item.getId());
            // 材料成本
            orderSkuProductionCost.setMaterialCost(item.getUnitPrice().multiply(item.getQuantity()));
            // 辅料成本暂时没有默认为零
            orderSkuProductionCost.setAuxiliaryMaterialCost(BigDecimal.ZERO);
            // 产品包材成本
            List<OrderSkuBom> bomList = orderSkuBomMap.get(item.getId());
            if (bomList == null) {
                bomList = new ArrayList<>();
            }
            BigDecimal productPackagingMaterialCost = bomList.stream()
                    .map(orderSkuBom -> orderSkuBom.getQuantity()
                            .multiply(orderSkuBom.getUnitPrice())
                            .multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            orderSkuProductionCost.setProductPackagingMaterialCost(productPackagingMaterialCost);
            // 物流包材成本
            orderSkuProductionCost.setLogisticsPackagingMaterialCost(item.getDeliveryMaterialsFee()
                    .multiply(item.getQuantity()));
            orderSkuProductionCost.setTotalAmount(
                    orderSkuProductionCost.getMaterialCost()
                            .add(orderSkuProductionCost.getAuxiliaryMaterialCost())
                            .add(orderSkuProductionCost.getProductPackagingMaterialCost())
                            .add(orderSkuProductionCost.getLogisticsPackagingMaterialCost()));
            return orderSkuProductionCost;
        }).collect(Collectors.toList());
        orderSkuProductionCostService.editLinked(orderSkuProductionCostList, OrderSkuProductionCost::getOrderId, dto.getId());

        // 是委外订单并且没有包材时，修改订单状态为生产中，并生成生产任务和工单
        if (ObjectUtil.equals(dto.getStatus(), OrderStatusEnum.STOCK_PREPARATION.getKey())
                && ObjectUtil.equals(dto.getType(), 2)
                && ObjectUtil.isEmpty(orderSkuBomList)) {
            dto.setStatus(OrderStatusEnum.IN_PRODUCTION.getKey());
            // 生成工单
            productionWorkOrderService.add(dto.getId());
            this.update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey()).eq(BaseIdPo::getId, dto.getId()));
        }

        // 更新订单产品包装信息
        List<OrderPackageBomDto> tempOrderPackageBomList = dto.getOrderPackageBomList();
        if (tempOrderPackageBomList == null) {
            tempOrderPackageBomList = new ArrayList<>();
        }
        List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .map(item -> (OrderPackageBom) item)
                .collect(Collectors.toList());
        orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, dto.getId());

        // 订单操作日志
        if (dto.getUpdateType() != null) {
            OrderOperatingLog orderOperatingLog = new OrderOperatingLog();
            orderOperatingLog.setType(dto.getUpdateType());
            orderOperatingLog.setOrderId(dto.getId());
            orderOperatingLog.setOrderCode(dto.getCode());
            orderOperatingLogService.save(orderOperatingLog);
        }

        // 更新订单产品不干胶图稿
        for (OrderSkuDto item : orderSkuList) {
            ObsFileUtil.editFile(item.getSelfAdhesiveStickerFile(), item.getId());
        }

        // 更新订单附件
        ObsFileUtil.editFile(dto.getFileList(), dto.getId());

    }

    @DSTransactional
    @Override
    public void confirmation(OrderInfoDto dto) {
        OrderInfo order = getById(dto.getId());

        if (ObjectUtil.notEqual(order.getStatus(), OrderStatusEnum.DRAFT.getKey())
                && ObjectUtil.notEqual(order.getStatus(), OrderStatusEnum.UNDER_REVIEW.getKey())) {
            throw new ServiceException("订单不在草稿或待确认状态");
        }

        // 自主订单需要锁定库存
        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();
        if (Objects.equals(dto.getType(), 1)) {
            // 先判断成品是否能锁定
            if (inventoryFinishedService.lockNoSourceStorage(orderSkuList)) {
                dto.setStockType(StatusConstant.YES);
            } else {
                lockStorage(orderSkuList);
                dto.setStockType(StatusConstant.NO);
            }
            dto.setLockStorage(StatusConstant.YES);
        }

        // 更新订单
        dto.setStatus(OrderStatusEnum.STOCK_PREPARATION.getKey());
        // 订单为委外订单时，订单分类为委外订单，订单费用修改
        if (Objects.equals(dto.getType(), 2)) {
            dto.setClassify(OrderClassifyEnum.OUTSOURCE_ORDER.getKey());
            if (!Objects.equals(dto.getSource(), 1)) {
                dto.setLssueFee(BigDecimal.ZERO);
            }
            dto.setProductTotalAmount(BigDecimal.ZERO);
            dto.setDeliveryMaterialsFee(BigDecimal.ZERO);
            dto.setPackingLabor(dto.getPackingLabor().multiply(new BigDecimal(2)));
            dto.setTotalAmount(dto.getProductTotalAmount()
                    .add(dto.getCustomProcessingFee())
                    .add(dto.getLssueFee())
                    .add(dto.getDeliveryMaterialsFee())
                    .add(dto.getPackingLabor())
                    .add(dto.getPackagingMaterialCost())
                    .add(dto.getProofingFee())
                    .add(dto.getManagementFee()));
            // 清空订单快递包材
            dto.setOrderPackageBomList(null);
        } else if (Objects.equals(dto.getType(), 1)) {
            // 拥有采购角色的账号创建的自主订单为采购订单
            Long userId = dto.getCreateUser();
            Set<String> roleKeys = roleService.selectRolePermissionByUserId(userId);
            if (!Collections.disjoint(roleKeys, Arrays.asList("sypurchasing", "purchasingOfficer", "bzpurchasing"))) {
                dto.setClassify(OrderClassifyEnum.PURCHASE_ORDER.getKey());
            }
        }
        this.updateById(dto);

        // 更新订单产品
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .peek(item -> item.setStockPreparationStatus(StatusConstant.NO))
                .peek(item -> {
                    // 委外订单修改金额
                    if (ObjectUtil.equals(dto.getType(), 2)) {
                        item.setUnitPrice(BigDecimal.ZERO);
                        item.setLssueFee(BigDecimal.ZERO);
                        item.setDeliveryMaterialsFee(BigDecimal.ZERO);
                        item.setPackingLabor(item.getPackingLabor().multiply(new BigDecimal(2)));
                    }
                })
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.editLinked(tempOrderSkuList, OrderSku::getOrderId, dto.getId());

        // 更新包材
        List<OrderSkuBom> orderSkuBomList = orderSkuList.stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(dto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
        orderSkuBomService.editLinked(orderSkuBomList, OrderSkuBom::getOrderId, dto.getId());

        // 新增订单产品生产成本
        Map<Long, List<OrderSkuBom>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        List<OrderSkuProductionCost> orderSkuProductionCostList = tempOrderSkuList.stream().map(item -> {
            OrderSkuProductionCost orderSkuProductionCost = new OrderSkuProductionCost();
            orderSkuProductionCost.setOrderId(item.getOrderId());
            orderSkuProductionCost.setOrderSkuId(item.getId());
            // 材料成本
            orderSkuProductionCost.setMaterialCost(item.getUnitPrice().multiply(item.getQuantity()));
            // 辅料成本暂时没有默认为零
            orderSkuProductionCost.setAuxiliaryMaterialCost(BigDecimal.ZERO);
            // 产品包材成本
            List<OrderSkuBom> bomList = orderSkuBomMap.get(item.getId());
            if (bomList == null) {
                bomList = new ArrayList<>();
            }
            BigDecimal productPackagingMaterialCost = bomList.stream()
                    .map(orderSkuBom -> orderSkuBom.getQuantity()
                            .multiply(orderSkuBom.getUnitPrice())
                            .multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            orderSkuProductionCost.setProductPackagingMaterialCost(productPackagingMaterialCost);
            // 物流包材成本
            orderSkuProductionCost.setLogisticsPackagingMaterialCost(item.getDeliveryMaterialsFee()
                    .multiply(item.getQuantity()));
            orderSkuProductionCost.setTotalAmount(
                    orderSkuProductionCost.getMaterialCost()
                            .add(orderSkuProductionCost.getAuxiliaryMaterialCost())
                            .add(orderSkuProductionCost.getProductPackagingMaterialCost())
                            .add(orderSkuProductionCost.getLogisticsPackagingMaterialCost()));
            return orderSkuProductionCost;
        }).collect(Collectors.toList());
        orderSkuProductionCostService.editLinked(orderSkuProductionCostList, OrderSkuProductionCost::getOrderId, dto.getId());

        // 是委外订单并且没有包材时，修改订单状态为生产中，并生成生产任务和工单
        if (ObjectUtil.equals(dto.getStatus(), OrderStatusEnum.STOCK_PREPARATION.getKey())
                && ObjectUtil.equals(dto.getType(), 2)
                && ObjectUtil.isEmpty(orderSkuBomList)) {
            dto.setStatus(OrderStatusEnum.IN_PRODUCTION.getKey());
            // 生成工单
            productionWorkOrderService.add(dto.getId());
            this.update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey()).eq(BaseIdPo::getId, dto.getId()));
        }

        // 更新订单产品包装信息
        List<OrderPackageBomDto> tempOrderPackageBomList = dto.getOrderPackageBomList();
        if (tempOrderPackageBomList == null) {
            tempOrderPackageBomList = new ArrayList<>();
        }
        List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .map(item -> (OrderPackageBom) item)
                .collect(Collectors.toList());
        orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, dto.getId());

        // 更新产品不干胶图稿
        for (OrderSkuDto item : orderSkuList) {
            ObsFileUtil.editFile(item.getSelfAdhesiveStickerFile(), item.getId());
        }

        // 更新附件
        ObsFileUtil.editFile(dto.getFileList(), dto.getId());

        // 发送mq下载临时图稿
        sendMq(dto);

    }

    @DSTransactional
    @Override
    public synchronized void delete(Long id, boolean saveLog) {
        OrderInfo orderInfo = getById(id);

        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }

        if (Objects.equals(orderInfo.getDelFlag(), StatusConstant.NOT_DELETED)
                && orderInfo.getStatus() > OrderStatusEnum.STOCK_PREPARATION.getKey()) {
            throw new ServiceException("订单正在生产中，无法直接删除，需要走审批流程");
        }

        unLockStorage(orderInfo);

        // 删除订单回滚成品仓库存
        inventoryFinishedService.removeOrder(id);

        // 删除订单以及订单关联数据
        delete(id);

        if (saveLog) {
            OrderOperatingLog orderOperatingLog = new OrderOperatingLog();
            orderOperatingLog.setType(30);
            orderOperatingLog.setOrderId(id);
            orderOperatingLog.setOrderCode(orderInfo.getCode());
            orderOperatingLogService.save(orderOperatingLog);
        }

    }

    @Override
    public SkuSpecPriceVo getSkuSpecPrice(SkuSpecPriceDto dto) {

        // 查询单价
        Long skuSpecId = dto.getSkuSpecId();
        SkuSpec skuSpec = skuSpecService.getById(skuSpecId);
        Assert.notNull(skuSpec, "未知sku规格");
        Long bomSpecId = skuSpec.getBomSpecId();
        Assert.notNull(bomSpecId, "sku未绑定bom");
        BomSpec bomSpec = bomSpecService.getById(bomSpecId);
        Assert.notNull(bomSpec, "未知bom规格");
        Bom bom = bomService.getById(bomSpec.getBomId());
        Assert.notNull(bom, "未知bom");

        // 其他价格查询
        Long deptId = SecurityUtils.getDeptId();
        Department department = departmentService.getById(deptId);
        Assert.notNull(department, "未知事业部");
        String priceBillingStandardIdStr = department.getPriceBillingStandardId();
        Assert.notBlank(priceBillingStandardIdStr, "事业部未绑定加工计费标准");

        List<String> priceBillingStandardIdList = Arrays.stream(priceBillingStandardIdStr.split(",")).collect(Collectors.toList());
        List<PriceBillingStandard> priceBillingStandardList = priceBillingStandardService.listByIds(priceBillingStandardIdList);

        List<PriceBillingStandardDetail> priceBillingStandardDetailList = null;

        if (StrUtil.isBlank(bom.getSpecies())) {
            priceBillingStandardDetailList = Collections.emptyList();
        } else {
            for (PriceBillingStandard priceBillingStandard : priceBillingStandardList) {
                if (Objects.equals(bom.getSpecies(), priceBillingStandard.getSpecies())) {
                    priceBillingStandardDetailList = priceBillingStandardDetailService.list(
                            q -> q.eq(PriceBillingStandardDetail::getPriceBillingStandardId, priceBillingStandard.getId()));
                    break;
                }
            }

            if (priceBillingStandardDetailList == null) {
                throw new ServiceException("没有找到加工计费标准");
            }
        }

        // 赋值
        SkuSpecPriceVo skuSpecPriceVo = new SkuSpecPriceVo();
        skuSpecPriceVo.setUnitPrice(bomSpec.getInternalSellingPrice());
        skuSpecPriceVo.setCustomProcessingFee(BigDecimal.ZERO);
        skuSpecPriceVo.setLssueFee(BigDecimal.ZERO);
        skuSpecPriceVo.setDeliveryMaterialsFee(BigDecimal.ZERO);
        skuSpecPriceVo.setPackingLabor(BigDecimal.ZERO);
        skuSpecPriceVo.setManagementFee(BigDecimal.ZERO);

        for (PriceBillingStandardDetail priceBillingStandardDetail : priceBillingStandardDetailList) {
            String chargeItem = priceBillingStandardDetail.getChargeItem();
            BigDecimal chargePrice = ObjectUtil.defaultIfNull(priceBillingStandardDetail.getChargePrice(), BigDecimal.ZERO);
            switch (chargeItem) {
                case "40":
                    if (bomSpec.getWidth().compareTo(BigDecimal.valueOf(90)) > 0) {
                        skuSpecPriceVo.setPackingLabor(new BigDecimal("1.5"));
                    } else {
                        skuSpecPriceVo.setPackingLabor(chargePrice);
                    }
                    break;
                case "50":
                    skuSpecPriceVo.setLssueFee(chargePrice);
                    break;
                case "60":
                    skuSpecPriceVo.setDeliveryMaterialsFee(chargePrice);
                    break;
                case "70":
                    skuSpecPriceVo.setManagementFee(chargePrice);
                    break;
                default:
                    if (Objects.equals(chargeItem, skuSpec.getMachinedPanel())
                            && dto.getQuantity().compareTo(priceBillingStandardDetail.getSectionMin()) >= 0
                            && dto.getQuantity().compareTo(priceBillingStandardDetail.getSectionMax()) < 0) {
                        skuSpecPriceVo.setCustomProcessingFee(chargePrice);
                        skuSpecPriceVo.setCustomProcessingType(chargeItem);
                    }
            }
        }

        return skuSpecPriceVo;
    }

    @Override
    public String addExceptionType(String exceptionTypeStr, Integer addExceptionType) {
        if (StrUtil.isBlank(exceptionTypeStr) || exceptionTypeStr.equals("0")) {
            return addExceptionType.toString();
        }

        if (Arrays.asList(exceptionTypeStr.split(",")).contains(addExceptionType.toString())) {
            return exceptionTypeStr;
        }

        return exceptionTypeStr + "," + addExceptionType;
    }

    @LogicIgnore
    @Override
    public Page<OrderInfoVo> deletedOrderPage(OrderSelectDto dto) {
        IWrapper<OrderInfo> wrapper = getWrapper();
        wrapper.orderByDesc("o", OrderInfo::getId);
        wrapper.eq("d", Department::getId, dto.getDepartmentId());
        wrapper.like("o", OrderInfo::getCode, dto.getCode());
        wrapper.like("o", OrderInfo::getWlnCode, dto.getWlnCode());
        wrapper.eq("o", OrderInfo::getDelFlag, StatusConstant.DELETED);

        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @LogicIgnore
    @Override
    public OrderInfoVo deletedOrderDetail(BaseSelectDto dto) {
        return detail(dto.getId());
    }

    @DSTransactional
    @Override
    public void turnToRegularOrder(Long id) {
        OrderInfo orderInfo = getById(id);
        String exceptionType = orderInfo.getExceptionType();

        if (Objects.equals(exceptionType, OrderExceptionTypeEnum.NORMAL.getKey().toString())) {
            return;
        }

        for (String s : exceptionType.split(",")) {
            OrderExceptionTypeEnum orderExceptionTypeEnum = OrderExceptionTypeEnum.getEnum(Convert.toInt(s));
            if (ObjectUtil.notEqual(orderExceptionTypeEnum, OrderExceptionTypeEnum.ORDER_CLOSURE)
                    && ObjectUtil.notEqual(orderExceptionTypeEnum, OrderExceptionTypeEnum.ORDER_ABORT)
                    && ObjectUtil.notEqual(orderExceptionTypeEnum, OrderExceptionTypeEnum.ORDER_EXCEPTION_HANDLING)) {
                throw new ServiceException(orderExceptionTypeEnum.getValue() + "，无法转为正常订单");
            }
        }

        if (ObjectUtil.equal(orderInfo.getStatus(), OrderStatusEnum.STOCK_PREPARATION.getKey())) {
            List<OrderSku> list = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderInfo.getId()));
            // 先判断成品是否能锁定
            if (inventoryFinishedService.lockNoSourceStorage(list)) {
                orderInfo.setStockType(StatusConstant.YES);
            } else {
                lockStorage(list);
                orderInfo.setStockType(StatusConstant.NO);
            }
            update(q -> q
                    .eq(BaseIdPo::getId, orderInfo.getId())
                    .set(OrderInfo::getLockStorage, StatusConstant.YES)
                    .set(OrderInfo::getStockType, orderInfo.getStockType()));
        }

        update(q -> q
                .eq(BaseIdPo::getId, orderInfo.getId())
                .set(OrderInfo::getExceptionType, OrderExceptionTypeEnum.NORMAL.getKey().toString())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        // 转trade
        if (Arrays.asList(exceptionType.split(",")).contains(OrderExceptionTypeEnum.ORDER_CLOSURE.getKey().toString())) {
            // 订单分类修改为无理由订单
            update(q -> q
                    .eq(BaseIdPo::getId, orderInfo.getId())
                    .set(OrderInfo::getClassify, OrderClassifyEnum.NO_REASON_ORDER.getKey())
            );
        }

    }

    @DSTransactional
    @Override
    public synchronized void deleteAndStore(Long id) {

        OrderInfo orderInfo = getById(id);

        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }

        unLockStorage(orderInfo);

        if (orderInfo.getStatus() <= OrderStatusEnum.STOCK_PREPARATION.getKey()) {
            delete(id);
            return;
        }

        // 无理由订单完成时间超过当天不能删除
        if (Objects.equals(orderInfo.getClassify(), OrderClassifyEnum.NO_REASON_ORDER.getKey())
                && Objects.equals(orderInfo.getStatus(), OrderStatusEnum.COMPLETION_PRODUCTION.getKey())) {
            if (DateUtil.beginOfDay(new Date()).compareTo(orderInfo.getShippingTime()) > 0) {
                throw new ServiceException("无理由订单删除只支持当天！");
            }
        }

        // 获取订单商品主材和包材
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.eq(OrderSkuBom::getOrderId, id));
        List<OrderPackageBom> orderPackageBomList = orderPackageBomService.list(q -> q.eq(OrderPackageBom::getOrderId, id));

        // 成品退料
        if (Objects.equals(orderInfo.getStockType(), StatusConstant.YES)) {
            inventoryFinishedService.noSourceInWarehousing(orderSkuList);
            // 快递包材退料
            Map<Long, InOutStorageBom> inStorageBomMap = orderPackageBomList.stream()
                    .map(item -> {
                        InOutStorageBom inStorageBom = new InOutStorageBom();
                        inStorageBom.setBomSpecId(item.getBomSpecId());
                        inStorageBom.setQuantity(item.getQuantity());
                        return inStorageBom;
                    }).collect(Collectors.toMap(
                            InOutStorageBom::getBomSpecId,
                            Function.identity(),
                            (v1, v2) -> {
                                v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                                return v1;
                            }
                    ));
            InOutStorageDto packageBomInStorageDto = new InOutStorageDto();
            packageBomInStorageDto.setType(InOutTypeEnum.IN.getKey());
            packageBomInStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
            packageBomInStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
            packageBomInStorageDto.setDepartmentId(DepartmentConstant.SD_SPORTS);
            packageBomInStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
            packageBomInStorageDto.setRemark("删除订单：" + orderInfo.getCode()
                    + (StrUtil.isBlank(orderInfo.getWlnCode()) ? StringPool.EMPTY : "（" + orderInfo.getWlnCode() + "）")
                    + " 包材退料入库");
            packageBomInStorageDto.setInOutStorageBomList(new ArrayList<>(inStorageBomMap.values()));
            inOutStorageService.add(packageBomInStorageDto);

            // 删除订单回滚成品仓库存
            inventoryFinishedService.removeOrder(id);

            delete(id);
            OrderOperatingLog orderOperatingLog = new OrderOperatingLog();
            orderOperatingLog.setType(30);
            orderOperatingLog.setOrderId(id);
            orderOperatingLog.setOrderCode(orderInfo.getCode());
            orderOperatingLogService.save(orderOperatingLog);
            return;
        }

        Map<Long, OrderSku> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));
        // 筛选掉赠品sku
        List<Long> skuSpecIds = orderSkuList.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Set<Long> giftSkuSpecIds = skuSpecService.getGiftIdListByIdList(skuSpecIds);

        // 合并主材和包材
        List<InOutStorageBom> inOutStorageBomList = orderSkuList.stream()
                .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                .map(item -> {
                    InOutStorageBom inOutStorageBom = new InOutStorageBom();
                    inOutStorageBom.setBomSpecId(item.getBomSpecId());
                    inOutStorageBom.setQuantity(item.getQuantity());
                    return inOutStorageBom;
                }).collect(Collectors.toList());

        for (OrderSkuBom orderSkuBom : orderSkuBomList) {
            OrderSku orderSku = orderSkuMap.get(orderSkuBom.getOrderSkuId());
            InOutStorageBom inOutStorageBom = new InOutStorageBom();
            inOutStorageBom.setBomSpecId(orderSkuBom.getBomSpecId());
            inOutStorageBom.setQuantity(orderSkuBom.getQuantity().multiply(orderSku.getQuantity()));
            inOutStorageBomList.add(inOutStorageBom);
        }

        // 无理由订单不会出库快递包材，不需要退料
        if (!Objects.equals(orderInfo.getClassify(), OrderClassifyEnum.NO_REASON_ORDER.getKey())) {
            for (OrderPackageBom orderPackageBom : orderPackageBomList) {
                InOutStorageBom inOutStorageBom = new InOutStorageBom();
                inOutStorageBom.setBomSpecId(orderPackageBom.getBomSpecId());
                inOutStorageBom.setQuantity(orderPackageBom.getQuantity());
                inOutStorageBomList.add(inOutStorageBom);
            }
        }

        // 合并数量
        Map<Long, InOutStorageBom> map = inOutStorageBomList.stream().collect(Collectors.toMap(
                InOutStorageBom::getBomSpecId,
                Function.identity(),
                (v1, v2) -> {
                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                    return v1;
                }
        ));
        inOutStorageBomList = new ArrayList<>(map.values());

        // 区分半成品仓和包材仓库
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(map.keySet());

        // 主材从半成品仓入库
        List<InOutStorageBom> semiFinishedProductInOutStorageBomList = inOutStorageBomList
                .stream()
                .filter(item -> {
                    BomSpecBo tempBomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    if (tempBomSpecBo == null) {
                        return false;
                    }
                    return ObjectUtil.equal(tempBomSpecBo.getClassifyParentId(), 1L);
                })
                .collect(Collectors.toList());
        InOutStorageDto semiFinishedProductInOutStorageDto = new InOutStorageDto();
        semiFinishedProductInOutStorageDto.setType(InOutTypeEnum.IN.getKey());
        semiFinishedProductInOutStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
        semiFinishedProductInOutStorageDto.setWarehouseId(WarehouseConstant.SEMI_FINISHED_PRODUCT);
        semiFinishedProductInOutStorageDto.setDepartmentId(orderInfo.getOutDepartmentId());
        semiFinishedProductInOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
        semiFinishedProductInOutStorageDto.setRemark("删除订单：" + orderInfo.getCode()
                + (StrUtil.isBlank(orderInfo.getWlnCode()) ? StringPool.EMPTY : "（" + orderInfo.getWlnCode() + "）")
                + " 主材退料入库");
        semiFinishedProductInOutStorageDto.setInOutStorageBomList(semiFinishedProductInOutStorageBomList);
        inOutStorageService.add(semiFinishedProductInOutStorageDto);

        // 包材从包材仓出库
        List<InOutStorageBom> packagingMaterialInOutStorageBomList = inOutStorageBomList
                .stream()
                .filter(item -> {
                    BomSpecBo tempBomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    if (tempBomSpecBo == null) {
                        return true;
                    }
                    return ObjectUtil.notEqual(tempBomSpecBo.getClassifyParentId(), 1L);
                })
                .collect(Collectors.toList());
        InOutStorageDto packagingMaterialInOutStorageDto = new InOutStorageDto();
        packagingMaterialInOutStorageDto.setType(InOutTypeEnum.IN.getKey());
        packagingMaterialInOutStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
        packagingMaterialInOutStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
        packagingMaterialInOutStorageDto.setDepartmentId(orderInfo.getOutDepartmentId());
        packagingMaterialInOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
        packagingMaterialInOutStorageDto.setRemark("删除订单：" + orderInfo.getCode()
                + (StrUtil.isBlank(orderInfo.getWlnCode()) ? StringPool.EMPTY : "（" + orderInfo.getWlnCode() + "）")
                + " 包材退料入库");
        packagingMaterialInOutStorageDto.setInOutStorageBomList(packagingMaterialInOutStorageBomList);
        inOutStorageService.add(packagingMaterialInOutStorageDto);

        // 删除订单回滚成品仓库存
        inventoryFinishedService.removeOrder(id);

        delete(id);

        OrderOperatingLog orderOperatingLog = new OrderOperatingLog();
        orderOperatingLog.setType(30);
        orderOperatingLog.setOrderId(id);
        orderOperatingLog.setOrderCode(orderInfo.getCode());
        orderOperatingLogService.save(orderOperatingLog);
    }

    @Override
    public CompareVo compare(OrderSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(999999999);
        Page<OrderInfoVo> page = getPage(dto);
        List<OrderInfoVo> records = page.getRecords();
        List<String> wlnCodeList = records.stream().map(OrderInfo::getWlnCode).collect(Collectors.toList());

        String compareStr = dto.getCompareStr();
        List<String> list = Arrays.asList(compareStr.split("\n"));

        List<String> systemMissingOrderList = list.stream().filter(item -> !wlnCodeList.contains(item)).collect(Collectors.toList());
        List<String> systemSurplusOrderList = wlnCodeList.stream().filter(item -> !list.contains(item)).collect(Collectors.toList());

        CompareVo compareVo = new CompareVo();
        compareVo.setSystemMissingOrderList(systemMissingOrderList);
        compareVo.setSystemSurplusOrderList(systemSurplusOrderList);

        return compareVo;
    }

    @Override
    public List<BomSpecOrderConsumptionBo> getOrderBomSpecOutStorageQuantity(List<Long> bomSpecIds, Date startDate, Date endDate) {
        // 根据对账时间获取有对账单的订单
        List<StatementOfAccount> list = statementOfAccountService.list(q -> q.between(StatementOfAccount::getTimePeriod, startDate, endDate));
        List<Long> accountIds = list.stream().map(StatementOfAccount::getId).collect(Collectors.toList());


        IWrapper<OrderInfo> wrapper = getWrapper();
        wrapper.in("o", OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey(), OrderStatusEnum.COMPLETION_PRODUCTION.getKey());
        wrapper.in("o", OrderInfo::getStatementOfAccountId, accountIds);

        // 查询主材消耗数量
        wrapper.groupBy("os.bom_spec_id", "soa.time_period");
        wrapper.in("os", OrderSku::getBomSpecId, bomSpecIds);
        List<BomSpecOrderConsumptionBo> orderSkuOutStorageList = this.baseMapper.getOrderSkuOutStorageQuantity(wrapper);

        wrapper = IWrapper.getWrapper();
        wrapper.in("o", OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey(), OrderStatusEnum.COMPLETION_PRODUCTION.getKey());
        wrapper.in("o", OrderInfo::getStatementOfAccountId, accountIds);
        wrapper.groupBy("osb.bom_spec_id", "soa.time_period");
        // 查询包材消耗数量
        wrapper.in("osb", OrderSkuBom::getBomSpecId, bomSpecIds);
        List<BomSpecOrderConsumptionBo> orderSkuBomOutStorageList = this.baseMapper.getOrderSkuBomOutStorageQuantity(wrapper);
        List<BomSpecOrderConsumptionBo> bomOutStorageList = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(orderSkuOutStorageList)) {
            bomOutStorageList.addAll(orderSkuOutStorageList);
        }
        if (ObjectUtil.isNotEmpty(orderSkuBomOutStorageList)) {
            bomOutStorageList.addAll(orderSkuBomOutStorageList);
        }

        return bomOutStorageList;
    }

    @Override
    public String getOrderWlnCodeStr(OrderSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(999999999);
        Page<OrderInfoVo> page = getPage(dto);
        List<String> wlnCodeList = page.getRecords().stream().map(OrderInfo::getWlnCode).collect(Collectors.toList());
        return JSON.toJSONString(wlnCodeList);
    }

    @Override
    public String getOrderDeliveryCode() {
        return CodeEnum.SH_CODE.getCode();
    }

    @Override
    public void saveOrderDeliveryCode(OrderInfoDto dto) {
        OrderInfo orderInfo = this.getById(dto.getId());
        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }
        if (StrUtil.isBlank(dto.getDeliveryCode())) {
            throw new ServiceException("送货单号不能为空");
        }
        OrderInfo one = this.getOne(q -> q.eq(OrderInfo::getDeliveryCode, dto.getDeliveryCode()));
        if (one != null) {
            throw new ServiceException("送货单号重复，请重新生成！");
        }
        this.update(q -> q.set(OrderInfo::getDeliveryCode, dto.getDeliveryCode())
                .eq(BaseIdPo::getId, dto.getId()));
    }

    @Override
    public void suspendOrder(Long id) {
        OrderInfo orderInfo = this.getById(id);
        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }
        this.update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.SUSPEND.getKey())
                .eq(BaseIdPo::getId, id));
    }

    @Override
    public void cancelSuspendOrder(Long id) {
        OrderInfo orderInfo = this.getById(id);
        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }
        this.update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.STOCK_PREPARATION.getKey())
                .eq(BaseIdPo::getId, id));
    }

    @Override
    public StatementOrderClassifyTotalCountVo getOrderClassifyTotalCount(List<Long> statementIds) {
        IWrapper<OrderInfo> wrapper = getWrapper();
        wrapper.in("o", OrderInfo::getStatementOfAccountId, statementIds);
        return this.baseMapper.getOrderClassifyTotalCount(wrapper);
    }

    @Override
    public List<OutBomVo> returnBomList(Long id) {
        OrderInfo orderInfo = this.getById(id);
        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.eq(OrderSkuBom::getOrderId, id));
        List<OrderPackageBom> orderPackageBomList = orderPackageBomService.list(q -> q.eq(OrderPackageBom::getOrderId, id));
        Map<Long, OrderSku> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));
        if (Objects.equals(orderInfo.getClassify(), OrderClassifyEnum.OUTSOURCE_ORDER.getKey())) {
            orderSkuList = Collections.emptyList();
        }
        // 筛选掉赠品sku
        List<Long> skuSpecIds = orderSkuList.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Set<Long> giftSkuSpecIds = skuSpecService.getGiftIdListByIdList(skuSpecIds);

        Map<Long, OutBomVo> map = Stream.of(
                        // 主材
                        orderSkuList.stream()
                                .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                                .map(item -> {
                                    OutBomVo outBomVo = new OutBomVo();
                                    outBomVo.setBomSpecId(item.getBomSpecId());
                                    outBomVo.setOutQuantity(item.getQuantity());
                                    return outBomVo;
                                }),

                        // 包材
                        orderSkuBomList.stream().map(item -> {
                            OrderSku orderSku = orderSkuMap.get(item.getOrderSkuId());
                            OutBomVo outBomVo = new OutBomVo();
                            outBomVo.setBomSpecId(item.getBomSpecId());
                            outBomVo.setOutQuantity(item.getQuantity().multiply(orderSku.getQuantity()));
                            return outBomVo;
                        }),

                        // 快递包材
                        orderPackageBomList.stream().map(item -> {
                            OutBomVo outBomVo = new OutBomVo();
                            outBomVo.setBomSpecId(item.getBomSpecId());
                            outBomVo.setOutQuantity(item.getQuantity());
                            return outBomVo;
                        })
                )
                .flatMap(stream -> stream)
                .collect(Collectors.toMap(
                        OutBomVo::getBomSpecId,
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setOutQuantity(v1.getOutQuantity().add(v2.getOutQuantity()));
                            return v1;
                        })
                );

        // 合并sku主材和包材
        List<OutBomVo> outBomVoList = new ArrayList<>(map.values());

        // 区分半成品仓和包材仓库
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(map.keySet());

        for (OutBomVo outBomVo : outBomVoList) {
            Long bomSpecId = outBomVo.getBomSpecId();
            BomSpecBo bomSpecBo = bomSpecBoMap.get(bomSpecId);
            outBomVo.setBomSpecCode(bomSpecBo.getBomSpecCode());
            outBomVo.setBomSpecName(bomSpecBo.getBomSpecName());
            outBomVo.setClassifyId(bomSpecBo.getClassifyId());
            outBomVo.setClassifyName(bomSpecBo.getClassifyName());
            outBomVo.setClassifyParentId(bomSpecBo.getClassifyParentId());

            // 主材
            if (Objects.equals(bomSpecBo.getClassifyParentId(), 1L)) {
                outBomVo.setWarehouseId(WarehouseConstant.SEMI_FINISHED_PRODUCT);
                outBomVo.setWarehouseName("半成品仓");
            }
            // 包材
            else {
                outBomVo.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
                outBomVo.setWarehouseName("包材仓");
            }
        }
        return outBomVoList.stream()
                .sorted(Comparator.comparing(OutBomVo::getWarehouseId, Comparator.reverseOrder()).thenComparing(OutBomVo::getBomSpecCode))
                .collect(Collectors.toList());
    }

    @Override
    public List<OrderPackageBomVo> getSkuSpecPackageBomList(SkuSpecPackageBomDto dto) {
        List<SkuSpecPriceDto> skuSpecList = dto.getSkuSpecList();
        List<Long> skuSpecIds = skuSpecList.stream().map(SkuSpecPriceDto::getSkuSpecId).collect(Collectors.toList());
        Map<Long, List<SkuSpecLink>> skuSpecMap = skuSpecLinkService.mapKGroup(
                SkuSpecLink::getSkuSpecId,
                q -> q.in(SkuSpecLink::getSkuSpecId, skuSpecIds).eq(SkuSpecLink::getType, 2));
        if (ObjectUtil.isEmpty(skuSpecMap)) {
            return Collections.emptyList();
        }
        List<Long> bomSpecIds = skuSpecMap.values().stream()
                .flatMap(item -> item.stream().map(SkuSpecLink::getBomSpecId))
                .collect(Collectors.toList());
        Map<Long, BomSpec> bomSpecPriceMap = bomSpecService.mapKEntity(
                BaseIdPo::getId, q -> q.in(BaseIdPo::getId, bomSpecIds));

        List<OrderPackageBomVo> list = new ArrayList<>();
        for (SkuSpecPriceDto skuSpecPriceDto : skuSpecList) {
            List<SkuSpecLink> skuSpecLinkList = skuSpecMap.getOrDefault(skuSpecPriceDto.getSkuSpecId(), Collections.emptyList());
            List<OrderPackageBomVo> packageBomVoList = skuSpecLinkList.stream().map(item -> {
                BomSpec bomSpec = bomSpecPriceMap.get(item.getBomSpecId());
                OrderPackageBomVo orderPackageBomVo = new OrderPackageBomVo();
                orderPackageBomVo.setBomSpecName(bomSpec.getName());
                orderPackageBomVo.setBomSpecCode(bomSpec.getCode());
                orderPackageBomVo.setCostPrice(bomSpec.getCostPrice());
                orderPackageBomVo.setInternalSellingPrice(bomSpec.getInternalSellingPrice());
                orderPackageBomVo.setBomSpecId(item.getBomSpecId());
                orderPackageBomVo.setQuantity(item.getQuantity().multiply(skuSpecPriceDto.getQuantity()));
                return orderPackageBomVo;
            }).collect(Collectors.toList());
            list.addAll(packageBomVoList);
        }
        // 合并相同bom
        Map<Long, OrderPackageBomVo> bomVoMap = list.stream().collect(Collectors.toMap(
                OrderPackageBom::getBomSpecId,
                Function.identity(),
                (v1, v2) -> {
                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                    return v1;
                }));
        Map<String, BomSpec> bomSpecMap = bomSpecService.mapKEntity(BomSpec::getCode, q -> q.in(BomSpec::getCode, "40305028", "40305032", "40305033"));
        Long bomSpecId = bomSpecMap.get("40305028").getId();
        OrderPackageBomVo orderPackageBomVo = bomVoMap.get(bomSpecId);
        // 判断是否有装瑜伽砖快递袋，2-4个数量更换快递包材
        if (orderPackageBomVo == null
                || orderPackageBomVo.getQuantity().compareTo(BigDecimal.ONE) <= 0
                || orderPackageBomVo.getQuantity().compareTo(BigDecimal.valueOf(4)) > 0) {
            return new ArrayList<>(bomVoMap.values());
        }

        BomSpec bomSpec;
        if (orderPackageBomVo.getQuantity().compareTo(BigDecimal.valueOf(2)) == 0) {
            bomSpec = bomSpecMap.get("40305032");
        } else {
            bomSpec = bomSpecMap.get("40305033");
        }
        OrderPackageBomVo packageBomVo = new OrderPackageBomVo();
        packageBomVo.setBomSpecName(bomSpec.getName());
        packageBomVo.setBomSpecCode(bomSpec.getCode());
        packageBomVo.setCostPrice(bomSpec.getCostPrice());
        packageBomVo.setInternalSellingPrice(bomSpec.getInternalSellingPrice());
        packageBomVo.setBomSpecId(bomSpec.getId());
        packageBomVo.setQuantity(BigDecimal.ONE);
        bomVoMap.put(bomSpecId, packageBomVo);

        return new ArrayList<>(bomVoMap.values());
    }

    @Override
    public void editOrderPackageBom(OrderInfoDto dto) {
        OrderInfo orderInfo = this.getById(dto.getId());
        if (orderInfo == null) {
            throw new ServiceException("未知订单");
        }
        // 生产中和生产完成的订单快递包材归还入库
        if (ObjectUtil.equals(orderInfo.getStatus(), OrderStatusEnum.IN_PRODUCTION.getKey())
                || ObjectUtil.equals(orderInfo.getStatus(), OrderStatusEnum.COMPLETION_PRODUCTION.getKey())) {
            dto.setCode(orderInfo.getCode());
            dto.setWlnCode(orderInfo.getWlnCode());
            this.orderPackageBomInOutStorage(dto);
        }

        List<OrderPackageBomDto> tempOrderPackageBomList = dto.getOrderPackageBomList();
        BigDecimal totalAmount = orderInfo.getTotalAmount();
        BigDecimal oldDeliveryMaterialsFee = orderInfo.getDeliveryMaterialsFee();
        BigDecimal newDeliveryMaterialsFee = BigDecimal.ZERO;
        if (ObjectUtil.isEmpty(tempOrderPackageBomList)) {
            orderPackageBomService.remove(q -> q.eq(OrderPackageBom::getOrderId, orderInfo.getId()));
        } else {
            List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                    .peek(item -> item.setOrderId(dto.getId()))
                    .map(item -> (OrderPackageBom) item)
                    .collect(Collectors.toList());
            orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, orderInfo.getId());
            newDeliveryMaterialsFee = orderPackageBomList.stream()
                    .map(item -> item.getInternalSellingPrice().multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
        }
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderInfo.getId()));
        // 筛选掉赠品sku
        List<Long> skuSpecIds = orderSkuList.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Set<Long> giftSkuSpecIds = skuSpecService.getGiftIdListByIdList(skuSpecIds);
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                .collect(Collectors.toList());
        // sku的数量
        BigDecimal quantity = tempOrderSkuList.stream().map(OrderSku::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add);
        // sku快递包材费
        BigDecimal skuDeliveryMaterialsFee = newDeliveryMaterialsFee.divide(quantity, 3, RoundingMode.CEILING);
        tempOrderSkuList.forEach(item ->
                item.setDeliveryMaterialsFee(skuDeliveryMaterialsFee)
        );
        // 订单更新快递包材费
        OrderInfo updateOrderInfo = new OrderInfo();
        updateOrderInfo.setId(orderInfo.getId());
        updateOrderInfo.setDeliveryMaterialsFee(skuDeliveryMaterialsFee.multiply(quantity).setScale(2, RoundingMode.HALF_UP));
        updateOrderInfo.setTotalAmount(totalAmount.subtract(oldDeliveryMaterialsFee).add(updateOrderInfo.getDeliveryMaterialsFee()));
        this.updateById(updateOrderInfo);
        orderSkuService.updateBatchById(tempOrderSkuList);
        ObsFileUtil.editFile(dto.getOuterBoxSelfAdhesiveStickerFile(), dto.getId(), 1);
    }

    /**
     * 订单修改快递包材出入库操作
     * @param dto
     */
    private void orderPackageBomInOutStorage(OrderInfoDto dto) {
        List<OrderPackageBom> oldOrderPackageBomList = orderPackageBomService.list(q -> q.eq(OrderPackageBom::getOrderId, dto.getId()));
        List<OrderPackageBomDto> newOrderPackageBomList = dto.getOrderPackageBomList();

        // 旧bom
        Map<Long, BigDecimal> oldBomMap = oldOrderPackageBomList.stream()
                .collect(Collectors.toMap(
                        OrderPackageBom::getBomSpecId,
                        OrderPackageBom::getQuantity,
                        BigDecimal::add
                ));
        Map<Long, BigDecimal> newBomMap = newOrderPackageBomList.stream()
                .collect(Collectors.toMap(
                        OrderPackageBom::getBomSpecId,
                        OrderPackageBom::getQuantity,
                        BigDecimal::add
                ));
        // 查询所有bom进行对比
        Set<Long> bomIds = new HashSet<>(oldBomMap.keySet());
        bomIds.addAll(newBomMap.keySet());
        Map<Long, BigDecimal> outBomMap = new HashMap<>();
        Map<Long, BigDecimal> inBomMap = new HashMap<>();
        for (Long bomId : bomIds) {
            BigDecimal oldReturnQuantity = oldBomMap.get(bomId);
            BigDecimal newReturnQuantity = newBomMap.get(bomId);
            // 旧bom没有，出库库存，反之则归还库存
            if (oldReturnQuantity == null) {
                outBomMap.put(bomId, newReturnQuantity);
                continue;
            } else if (newReturnQuantity == null) {
                inBomMap.put(bomId, oldReturnQuantity);
                continue;
            }
            // 都有值，判断数量
            if (oldReturnQuantity.compareTo(newReturnQuantity) == 0) {
                continue;
            }
            // 旧bom大于新bom则归还入库，反之出库库存
            if (oldReturnQuantity.compareTo(newReturnQuantity) > 0) {
                inBomMap.put(bomId, oldReturnQuantity.subtract(newReturnQuantity));
            } else {
                outBomMap.put(bomId, newReturnQuantity.subtract(oldReturnQuantity));
            }
        }
        if (!outBomMap.isEmpty()) {
            // 包材从包材仓出库
            List<InOutStorageBom> outStorageBomList = outBomMap.entrySet().stream()
                    .map(item -> {
                        InOutStorageBom inOutStorageBom = new InOutStorageBom();
                        inOutStorageBom.setBomSpecId(item.getKey());
                        inOutStorageBom.setQuantity(item.getValue());
                        return inOutStorageBom;
                    })
                    .collect(Collectors.toList());
            InOutStorageDto inOutStorageDto = new InOutStorageDto();
            inOutStorageDto.setType(InOutTypeEnum.OUT.getKey());
            inOutStorageDto.setDetailType(OutDetailTypeEnum.PRODUCTION.getKey());
            inOutStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
            inOutStorageDto.setDepartmentId(DepartmentConstant.SD_SPORTS);
            inOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
            inOutStorageDto.setInOutStorageBomList(outStorageBomList);
            inOutStorageDto.setLockStorage(StatusConstant.NO);
            inOutStorageService.add(inOutStorageDto);
        }
        if (!inBomMap.isEmpty()) {
            // 包材归还入库
            List<InOutStorageBom> inOutStorageBomList = inBomMap.entrySet()
                    .stream()
                    .map(item -> {
                        InOutStorageBom inOutStorageBom = new InOutStorageBom();
                        inOutStorageBom.setBomSpecId(item.getKey());
                        inOutStorageBom.setQuantity(item.getValue());
                        return inOutStorageBom;
                    })
                    .collect(Collectors.toList());
            InOutStorageDto packagingMaterialInOutStorageDto = new InOutStorageDto();
            packagingMaterialInOutStorageDto.setType(InOutTypeEnum.IN.getKey());
            packagingMaterialInOutStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
            packagingMaterialInOutStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
            packagingMaterialInOutStorageDto.setDepartmentId(DepartmentConstant.SD_SPORTS);
            packagingMaterialInOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
            packagingMaterialInOutStorageDto.setRemark("订单产品快递包装修改：" + dto.getCode()
                    + (StrUtil.isBlank(dto.getWlnCode()) ? StringPool.EMPTY : "（" + dto.getWlnCode() + "）")
                    + " 包材归还入库");
            packagingMaterialInOutStorageDto.setInOutStorageBomList(inOutStorageBomList);
            inOutStorageService.add(packagingMaterialInOutStorageDto);
        }
    }

    /**
     * 订单导出
     * @param id
     */
    @Override
    public void orderExportExcel(Long id) {
        OrderInfo orderInfo = this.getById(id);
        List<OrderSku> list = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<Long> bomSpecIds = list.stream().map(OrderSku::getBomSpecId).collect(Collectors.toList());
        List<Long> skuSpecIds = list.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIds);
        Map<Long, SkuSpec> skuSpecMap = skuSpecService.byIdsToMap(skuSpecIds);
        List<OrderSku> orderPackageSkuList = list.stream()
                .filter(item -> !Objects.equals(bomSpecBoMap.get(item.getBomSpecId()).getClassifyParentId(), 1L))
                .collect(Collectors.toList());
        List<OrderSku> orderSkuList = list.stream()
                .filter(item -> Objects.equals(bomSpecBoMap.get(item.getBomSpecId()).getClassifyParentId(), 1L))
                .collect(Collectors.toList());
        BigDecimal orderPackageSku = orderPackageSkuList.stream()
                .map(item -> (item.getUnitPrice()
                        .add(item.getDeliveryMaterialsFee())
                        .add(item.getPackagingMaterialCost()))
                        .multiply(item.getQuantity()))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal orderPackagePrice = orderPackageSku.divide(BigDecimal.valueOf(orderSkuList.size()), 2, RoundingMode.HALF_UP);
        List<OrderSkuExportVo> exportVoList = orderSkuList.stream().map(item -> {
            BigDecimal subtotal = item.getUnitPrice()
                    .add(item.getDeliveryMaterialsFee())
                    .add(item.getPackagingMaterialCost());
            BomSpecBo bomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
            SkuSpec skuSpec = skuSpecMap.get(item.getSkuSpecId());
            OrderSkuExportVo orderSkuExportVo = new OrderSkuExportVo();
            orderSkuExportVo.setWlnCode(orderInfo.getWlnCode());
            orderSkuExportVo.setSkuSpecCode(skuSpec.getCode());
            orderSkuExportVo.setBomSpecCode(bomSpecBo.getBomSpecCode());
            orderSkuExportVo.setErpCode(item.getErpCode());
            orderSkuExportVo.setFeatureCode(item.getFeatureCode());
            orderSkuExportVo.setQuantity(item.getQuantity());
            orderSkuExportVo.setTotalAmount(subtotal.multiply(item.getQuantity()).add(orderPackagePrice).setScale(2, RoundingMode.HALF_UP));
            return orderSkuExportVo;
        }).collect(Collectors.toList());
        ExcelUtil.export(response, exportVoList, OrderSkuExportVo.class);
    }

    @Override
    public Page<OrderInfoVo> getOrderSelectListByCode(OrderSelectDto dto) {
        IWrapper<OrderInfo> wrapper = getWrapper();
        wrapper.orderByDesc("o", OrderInfo::getId);
        wrapper.and(q -> q
                .and(r -> r.eq("o", OrderInfo::getWlnStorageCode, "T007").ge("o", OrderInfo::getWlnStatus, 2))
                .or(r -> r.ne("o", OrderInfo::getWlnStorageCode, "T007").ge("o", OrderInfo::getWlnStatus, 1))
                .or(r -> r.eq("o", OrderInfo::getSource, 1))
        );
        wrapper.and(StrUtil.isNotBlank(dto.getCode()), q -> q
                .like("o", OrderInfo::getCode, dto.getCode())
                .or()
                .like("o", OrderInfo::getWlnCode, dto.getCode()));

        wrapper.eq("o", OrderInfo::getExceptionType, OrderExceptionTypeEnum.NORMAL.getKey().toString());

        Page<OrderInfoVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    /**
     * 删除订单以及订单关联数据
     */
    public void delete(Long id) {
        this.removeById(id);
        orderSkuService.remove(q -> q.eq(OrderSku::getOrderId, id));
        orderSkuBomService.remove(q -> q.eq(OrderSkuBom::getOrderId, id));
        orderPackageBomService.remove(q -> q.eq(OrderPackageBom::getOrderId, id));
        orderSkuProductionCostService.remove(q -> q.eq(OrderSkuProductionCost::getOrderId, id));
        productionWorkOrderService.remove(q -> q.eq(ProductionWorkOrder::getOrderId, id));

        ObsFileUtil.removeFile(id);
    }

    /**
     * 锁定库存
     */
    private void lockStorage(List<? extends OrderSku> orderSkuList) {
        List<Long> bomSpecIdList = orderSkuList.stream().map(OrderSku::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIdList);

        boolean result = inventoryService.lockStorage(orderSkuList.stream()
                .filter(item -> bomSpecBoMap.get(item.getBomSpecId()).getClassifyParentId().equals(1L))
                .collect(Collectors.toList())
        );

        if (!result) {
            throw new ServiceException("主材库存不足，无法确认订单");
        }

    }

    /**
     * 解锁库存
     */
    private void unLockStorage(OrderInfo orderInfo) {

        if (ObjectUtil.notEqual(orderInfo.getLockStorage(), StatusConstant.YES)) {
            return;
        }

        orderInfo.setLockStorage(StatusConstant.NO);
        update(q -> q.eq(BaseIdPo::getId, orderInfo.getId()).set(OrderInfo::getLockStorage, StatusConstant.NO));

        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderInfo.getId()));

        List<Long> bomSpecIdList = orderSkuList.stream().map(OrderSku::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIdList);
        // 筛选掉赠品sku
        List<Long> skuSpecIds = orderSkuList.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Set<Long> giftSkuSpecIds = skuSpecService.getGiftIdListByIdList(skuSpecIds);

        orderSkuList = orderSkuList.stream()
                .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                .filter(item -> bomSpecBoMap.get(item.getBomSpecId()).getClassifyParentId().equals(1L))
                .collect(Collectors.toList());

        if (Objects.equals(orderInfo.getStockType(), 1)) {
            inventoryFinishedService.unlockNoSourceStorage(orderSkuList);
        } else {
            inventoryService.unlockStorage(orderSkuList);
        }
    }

    private void sendMq(OrderInfoDto dto) {

        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();

        for (OrderSkuDto orderSkuDto : orderSkuList) {

            if (orderSkuDto.getArtworkLibraryId() == null || orderSkuDto.getArtworkLibraryId() == 0L) {
                continue;
            }

            TempArtworkMessage tempArtworkMessage = new TempArtworkMessage();
            tempArtworkMessage.setImgUrl(orderSkuDto.getBlueprint());
            tempArtworkMessage.setFileUrl(orderSkuDto.getProductionDocument());

            RabbitMqUtil.send(ArtworkConfig.DIRECT_EXCHANGE_NAME, ArtworkConfig.TEMP_ARTWORK_QUEUE_NAME, tempArtworkMessage);
        }
    }

}
